// clang-format off
/**
 *  为防止 Xcode 破坏 js 代码格式，请下载 http://www.conhar.com/wp-content/uploads/2014/10/JavaScript.xclangspec
 *  并保存到 ~/Library/Developer/Xcode/Specifications 目录下
 */

(function(window) {
    if (window.XXJS) {
        return;
    }
    /**
     * 定义JSB
     */
    var XXJS = {
        /**
         *jsb版本号
         *三位版本号：1.0.0，模块上传者能看到的部分为前两位
         *前两位标识Hybrid Api版本，最后一位标识分端bridge实现版本
         *如：Android和iOS当前jsb版本为1.0.1和1.0.2，表示当前两端的Hybrid Api都一致，
         *只是在内部bridge实现上Android迭代了1次，iOS迭代了2次
         */
        _version: '1.1.0',
        /**
         * {boolean} 页面信息及DOM加载状态，全部加载完成为true，默认完成为false
         */
        _isAllReady: true
    },
    /**
     * {object} 存储返回函数
     */
    responseCallbacks = {},
    /**
     * 初始化callbackid值变量
     */
    uniqueId = 1,
    /**
     * {Array}
     */
    sendMessageQueue = [],

    messageHandlers = {},
    /**
     * {boolean} 异步获取设备信息返回状态，返回成功为true，默认未返回false
     */
    _isDeviceReady = true,
    /**
     * {boolean} 异步获取环境信息返回状态，返回成功为true，默认未返回false
     */
    _isAccountReady = true,
    /**
     * {boolean} 异步获取账户信息返回状态，返回成功为true，默认未返回false
     */
    _isLocationReady = true,
    /**
     * {boolean} 异步获取位置信息返回状态，返回成功为true，默认未返回false
     */
    _isEnvReady = true,
    /**
     * {boolean} DOM结构加载状态，加载完成为true，默认未加载完成false
     */
    _isDomReady = true;

    // 将XXJS暴露到window下：全局可使用，对旧版本向下兼容
    window.XXJS = XXJS;

    /**
     * Debug开发中的问题
     * @param    {Function}  callback(error) 出错后的回调，默认是alert
     */
    XXJS.enableDebug = function(callback) {
        /**
         * @param errorMsg     {String}  错误信息
         * @param scriptURI    {String}  出错的文件
         * @param lineNum      {Long}    出错代码的行号
         * @param columnNum    {Long}    出错代码的列号
         */

        // console.log('Debug')
        window.onerror = function(errorMsg, scriptURI, lineNum, columnNum) {
            // 有callback的情况下，将错误信息传递到options.callback中
            if (typeof callback === 'function') {
                callback({
                    message: errorMsg,
                    script: scriptURI,
                    line: lineNum,
                    column: columnNum
                });
            } else {
                // 其他情况，都以alert方式直接提示错误信息
                var msgs = [];
                msgs.push("代码有错误。");
                msgs.push("\n错误信息：", errorMsg);
                msgs.push("\n出错文件：", scriptURI);
                msgs.push("\n出错位置：", lineNum + '行，' + columnNum + '列');
                alert(msgs.join(''));
            }
        }
    };

    XXJS.enableDebug(function(obj){
        XXJS._sendMessageToNative("_DebugJSErrorLogger", obj, null);
    });

    /**
     *  注册供 native 调用的 js 方法，可提供回调，js 处理完成后回调通知 native
     *
     *  @param jsHandlerName {string}        注册的 js bridge 方法名，native 通过此方法名调用
     *  @param handler       {function}      处理方法，接收从 native 传来的 data 和回调方法
     *         = function(data, callback){
     *              // do something with 'data' from native.
     *              // notice native task done with 'callback'
     *           }
     */
    XXJS.registerJSHandler = function(jsHandlerName, handler) {
        messageHandlers[jsHandlerName] = handler;
    }

    /**
     *  调用 native 提供的 js bridge 方法，处理完成后若有返回数据，根据返回数据执行 callback
     *
     *  @param nativeHandlerName {strign}    native 注册的 js bridge 方法名
     *  @param args              {object}    传递的参数值，类型为 js object
     *  @param callback          {function}  方法回调处理
     *
     */
    XXJS.callNativeHandler = function(nativeHandlerName, args, callback) {
        XXJS._sendMessageToNative(nativeHandlerName, args, callback);
    }

    /**
     *  用于替代addJavascriptInterface的替代方案，返回值通过修改该iframe src实现
     */
    XXJS._sendMessageToNative = function(method, args, callback) {
        var callbackId;
        if (callback && typeof callback == 'function') {
            callbackId = 'xinxin_' + (uniqueId++) + '_' + new Date().getTime();
//            alert(callbackId + "/<xinxin>")
            responseCallbacks[callbackId] = callback;
//alert(responseCallbacks + " responseCallbacks " + callback)
//            for (var i=0;i<responseCallbacks.length;i++){
//                 alert(responseCallbacks[i] + "<br>");
//             }
        }
        args = args || {};
        args['callbackId'] = callbackId;
        args = JSON.stringify(args);

        var iframe = document.createElement('iframe');
        iframe.style.display = 'none';
        document.body.appendChild(iframe);
        iframe.src = 'xxjsapi://_?' + 'method=' + method + '&args=' + encodeURIComponent(args);
        setTimeout(function() {
            iframe.parentNode.removeChild(iframe);
        }, 0);
    }

    /**
     * 全局回调函数，用于 js 调用 nativeHandler 时的回调处理
     */
    XXJS._sendMessageToNativeCallbackHandler = function(callbackId, responseData) {
        try {
//            for (var i=0;i<responseCallbacks.length;i++){
//                            alert(responseCallbacks[i] + "<br>");
//               }
//
//alert(responseData + "/<br>")

            var callback = responseCallbacks[callbackId];
            if (!callback) {
                return
            };

            callback(responseData);
            responseCallbacks[callbackId] = null;
        } catch (e) {
            alert(e);
        }
    }

    /**
     *
     *  全局函数，native 调用 js 方法的入口
     *  @param messageJSON native 传递的信息详情
     *
     */
    XXJS._handleMessageFromNative = function(messageJSON) {
        setTimeout(function _timeoutDispatchMessageFromNative() {
            var message = JSON.parse(messageJSON);
            var type = message.type;
            if (type === 'jsCallNativeCallback') {
                // js 调用 nativeHandler 时的回调处理
                XXJS._sendMessageToNativeCallbackHandler(message.callbackId, message.responseData);
            }
            else if (type === 'nativeCallJs') {
                // native 调用 jsHandler 的处理
                var callback;
                if (message.callbackId) {
                    var callbackId = message.callbackId;
                    callback = function(responseData) {
                        XXJS.callNativeHandler(callbackId, {responseData:responseData}, null);
                    };
                }

                var registeredJsHandler = messageHandlers[message.handlerName];
                try {
                    registeredJsHandler(message.data, callback);
                }
                catch (exception) {
                    console.log("MSTWebViewJavascriptBridge: WARNING: javascript handler threw.", message, exception);
                    callback({
                        code:-1,
                        msg:'调用 xxjs js 接口异常',
                        exception:exception,
                    })
                }
                if (!registeredJsHandler) {
                    console.log("MSTWebViewJavascriptBridge: WARNING: no handler for message from ObjC:", message);
                    callback({
                        code:-2,
                        msg:'调用 xxjs js 接口未实现',
                    });
                }
            }
        });
    }

    /**
     * 从Native获取设备信息
     */
    function _getDeviceInfo(callback) {
        XXJS.callNativeHandler("device.getDeviceInfo", null, callback);
    }

    _getDeviceInfo(function(obj) {
        XXJS.device = {
            // String 设备型号名称
            name: obj['name'],
            // String 设备系统平台
            platform: obj['platform'],
            // String 设备系统版本javascript:openUrl('http%3A%2F%2Fdeveloper.baidu.com%2Fwiki%2Findex.php%3Ftitle=docs%2Fcplat%2Fmedia%2Fface%2Fsdk')
            os: obj['os'],
            // int 屏幕宽
            screenWidth: obj['screenWidth'],
            // int 屏幕高
            screenHeight: obj['screenHeight']
        };

        _isDeviceReady = true;
    });

    /**
     * {Object} 环境相关信息
     */
    XXJS.env = XXJS.env || {};

    function _getEnvInfo(callback) {
        XXJS.callNativeHandler("app.getEnvInfo", null, callback);
    }
    _getEnvInfo(function(obj) {
        /* 分开写是避免异步返回值，覆盖下面env相关方法 */
        XXJS.env.cuid = obj['cuid'];
        XXJS.env.appName = obj['appName'];
        XXJS.env.appVersion = obj['appVersion'];
        XXJS.env.appChannel = obj['appChannel'];

        _isEnvReady = true;
    });

    /**
     * 设置账户信息的值
     */
    XXJS.account = XXJS.account || {};

    function setAccountInfo(obj) {
        /* 分开写是避免异步返回值，覆盖下面account相关方法 */
        XXJS.account.isLogin = obj['isLogin'];
        XXJS.account.uid = obj['uid'];
        XXJS.account.uName = obj['uName'];
        XXJS.account.displayName = obj['displayName'];
        XXJS.account.mobile = obj['mobile'];

        _isAccountReady = true;
    }

    /**
     * 登陆框
     * @param onSuccess {function}  登陆成功，回调函数
     * @param onFail   {function}  登陆失败，回调函数
     *  XXJS.account.login({
            onSuccess : function(){
                XXJS.account.toast.show('登陆成功！');
            },
            onFail : function(res){
                XXJS.account.toast.show('登陆失败！' + res.errmsg);
            }
        });
     */
    XXJS.account.login = function(options) {
        var _options = {
            onSuccess: function() {},
            onFail: function() {}
        };
        options = _extend(_options, options || {});

        /* 未登录 */
        XXJS.callNativeHandler('account.login', null, function(res) {
            /* 登陆成功 */
            if (res.errno == 0) {
                options.onSuccess();
            } else {
                /* 登陆失败 */
                var errObj = {};
                errObj.errno = res.errno;
                errObj.errmsg = res.errmsg;
                options.onFail(errObj);
            }
        });
    };

    /**
     * 从Native获取账户信息
     */
    function _getAccountInfo(callback) {
        // 从Native账户信息具体方法实现
        XXJS.callNativeHandler("account.getAccount", null, callback);
    }

    _getAccountInfo(function(obj) {
        setAccountInfo(obj);
    });

    /**
     * Native层监控账户信息变化，调用update函数，更新账户信息
     * 返回当前账户信息{object}
     */
    XXJS.account = XXJS.account || {};
    XXJS._updateAccount = function(obj) {
        setAccountInfo(obj);
    }

    function setLocationInfo(obj) {
        /* 分开写是避免异步返回值，覆盖下面location相关方法 */
        // boolean 是否有定位
        XXJS.location.hasLocation = obj['hasLocation'];

        if ('locationCity' in obj) {
            // String 城市码
            XXJS.location.cityCode = obj['locationCity']['city_code'];
            // String 城市名
            XXJS.location.cityName = obj['locationCity']['city_name'];
            // String 短城市名
            XXJS.location.shortCityName = obj['locationCity']['short_name'];
            // int 定位状态(-1:失败 0:空闲 1:正在定位 2:定位成功)
            XXJS.location.status = obj['locationCity']['status'];
            // String 城市url
            XXJS.location.cityUrl = obj['locationCity']['city_url'];
        }
        if ('currentCity' in obj) {
            // String 选中城市code
            XXJS.location.selectCityCode = obj['currentCity']['city_code'];
            // String 选中城市名称
            XXJS.location.selectCityName = obj['currentCity']['city_name'];
            // String 选中城市缩写
            XXJS.location.selectShortCityName = obj['currentCity']['short_name'];
            // String 选中城市url
            XXJS.location.selectCityUrl = obj['currentCity']['city_url'];
        }
        // String 纬度
        XXJS.location.latitude = obj['latitude'];
        // String 经度
        XXJS.location.longitude = obj['longitude'];
        // String 地址
        XXJS.location.address = obj['address'];
        // String 城市url
        XXJS.location.district = obj['district'];

        _isLocationReady = true;
    }

    function _getLocationInfo(callback) {
        // 从Native获取定位信息具体方法实现
        XXJS.callNativeHandler("location.getLocation", null, callback);
    }

    /**
     * 从Native获取定位信息
     */
    _getLocationInfo(function(obj) {
        setLocationInfo(obj);
    });


    /**
     * 对象简单继承，后面的覆盖前面的，继承深度：deep=1
     * @private
     */
    var _extend = function() {
        var result = {},
        obj, k;
        for (var i = 0, len = arguments.length; i < len; i++) {
            obj = arguments[i];
            if (typeof obj === 'object') {
                for (k in obj) {
                    obj[k] && (result[k] = obj[k]);
                }
            }
        }
        return result;
    };


    /**
     * 页面
     */
    (function($) {
        /********** 页面 **********/
        $.page = $.page || {};

        /**
         * 打开页面
         * @param url     {string} 需打开页面的url
         * @param params  {object} 需在打开页面用到的参数
         */
        $.page.start = function(url, params) {
            $.callNativeHandler('page.start', {
                url: url,
                params: params
            });
        };

        /**
         * 返回至上一个页面
         */
        $.page.back = function(delta) {
            $.callNativeHandler('page.back', {
                delta: delta
            });
        };

        /**
         * 获取客户端当前导航的栈（导航历史记录）
         */
        $.page.getHistory = function(options) {
            var _options = {
                onSuccess: function() {},
                onFail: function() {}
            };
            options = _extend(_options, options || {});

//            alert(options);

            $.callNativeHandler('page.getHistory', options, function(res) {
//            alert(res);
                if (res.errno == 0) {
                    // 获取客户端当前导航历史记录成功
                    options.onSuccess(res);
                } else {
                    // 获取客户端当前导航历史记录失败
                    var errObj = {};
                    errObj.errno = res.errno;
                    errObj.errmsg = res.errmsg;
                    options.onFail(errObj);
                }
            });
        };

        /**
         * 移除客户端导航历史记录的某一个页面（实现后退时跳过改页面）
         */
        $.page.removeHistoryItemAtIndex = function(options) {
            var _options = {
                index: -1,
                onSuccess: function() {},
                onFail: function() {}
            };
            options = _extend(_options, options || {});
            $.callNativeHandler('page.removeHistoryItemAtIndex', options, function(res) {
                if (res.errno == 0) {
                    // 删除页面成功
                    options.onSuccess(res);

                } else {
                    // 删除页面失败
                    var errObj = {};
                    errObj.errno = res.errno;
                    errObj.errmsg = res.errmsg;
                    options.onFail(errObj);
                }
            });
        };

        /**
         * 获取页面回传数据
         * @param callback 获取回传数据成功回调函数
         */
        $.page.getData = function(callback) {
            $.callNativeHandler('page.getData', null, callback);
        };
    })(XXJS);

    /**
     *  本地硬件功能
     */
    (function($) {
        $.hardware = $.hardware || {};

        /**
         *  调用本地二维码扫描界面，扫描完成后回调扫描结果
         *
         *  @param callback 扫描结果回调函数
         */
//        $.hardware.scanQRCode = function(callback) {
//            $.callNativeHandler('hardware.scanQRCode', null, callback);
//        };
//
//        /**
//         *  开启、关闭监听摇一摇
//         *
//         *  @param enable   true/false,
//         *  @param callback 摇一摇触发回调
//         */
//        $.hardware.shakeMotionListen = function(enable, callback) {
//            $.callNativeHandler('hardware.shakeMotionListen', {
//                enable:enable
//            }, callback);
//        };
    })(XXJS);

    /**
     * 位置信息，是否有定位/当前定位状态/获取定位信息/监听位置变化
     */
    (function($) {
        /********** 定位 **********/
        $.location = $.location || {};

        /**
         * Native层监控位置信息变化，调用update函数，更新位置信息
         * 返回当前位置信息{object}
         */
        $._updateLocation = function(newData) {
            setLocationInfo(newData);
        };
    })(XXJS);

    /**
     * UI组件，Toast/Dialog/Loading/Login/Title等
     */
    (function($) {
        /********** UI组组件 **********/
        $.ui = $.ui || {};

        /** Toast **/
        $.ui.toast = {
            /**
             * 显示Toast
             * @param {string} title     文本内容
             * @param {string} message   详情内容
             * @param {int}    duration  展示时长，单位：毫秒
             * @param {int}    type      toast 类型， 0为普通消息、1为警告信息、2为错误信息、3为其他信息
             */
            show: function(message, duration, title, type) {
                $.callNativeHandler('ui.toast', {
                    title: title,
                    message: message,
                    duration: duration,
                    type: 0,
                });
            }
        };

        /** 对话框 **/
        $.ui.dialog = {
            /**
             * @param options {object}
             *        options参数描述:
             *        @title        {string}   标题              可选，默认''
             *        @message      {string}   展示文本内容       可选，默认''
             *        @style        {string}   对话框样式         默认'alert', 可选'actionsheet'
             *        @actions      {array}    响应按钮文本       可选，默认'确定'
             *          item {
             *                  title:'按钮标题',
             *                  style:'default',  // 不传为 default, 可选 cancel, destructive
             *                  tag:2,
             *               }
             *        @onAction     {function}  点击按钮回调函数
             */
            show: function(options) {
                var _options = {
                    title: '',
                    message: '',
                    style: 'alert',
                    actions: null,
                    onAction: null,
                };
                options = _extend(_options, options || {});

                $.callNativeHandler('ui.dialog.show', options, function(res) {
                    options.onAction(res);
                });
            }
        };

        /** 加载中动画 **/
        $.ui.loading = {
            show: function(text, duration) {
                $.callNativeHandler('ui.loading.show', {
                    text: text,            // 不传时不展示文字
                    duration: duration,    // 不传或为0时，不主动消失
                });
            },
            hide: function() {
                $.callNativeHandler('ui.loading.hide');
            }
        }

        /** 分享 **/
        $.ui.share = {
            /**
             *  调用本地分享视图
             *
             *  @param options {object}
             *        options 参数描述
             *        @title    {string}   分享的标题，必填
             *        @detail   {string}   分享标题的补充，可选。（只在部分分享场景下出现<微信聊天新闻>）
             *        @imgUrl   {string}   分享带的图片链接，可选
             *        @url      {string}   分享的链接，可选
             *        @onSuccess {function} 分享组件调用成功回调
             *        @onFail    {function} 分享组件调用失败回调
             *        @callback  {function} 分享完成事件回调，在此处理『分享成功』『分享失败』
             *            eg: callback: function (result) {
             *                   // result.success     , 分享是否成功，成功为 'true'
             *                   // result.platform    , 分享到的平台，可取值 'weibo', 'wechat_friend', 'wechat_moments', 'qq_friend', 'qq_qzone'.
             *                   if (result.success === 'true')
             *                      XXJS.ui.toast.show('分享到 ' + result.platform + ' 成功');
             *                }
             */
            show: function(options) {
                var _options = {
                    title: '',
                    detail: '',
                    imgUrl: '',
                    url: '',
                    onSuccess: function() {},
                    onFail: function() {},
                    callback: function() {}
                };
                options = _extend(_options, options || {});

                if (options.callback && typeof options.callback == 'function') {
                    messageHandlers['ui.share.callbackHandler'] = options.callback;
                }

                $.callNativeHandler('ui.share.show', options, function(res){
                    if (res.errno == 0) {
                        // 分享调用成功
                        options.onSuccess();
                    } else {
                        /* 分享调用失败 */
                        var errObj = {};
                        errObj.errno = res.errno;
                        errObj.errmsg = res.errmsg;
                        options.onFail(errObj);
                    }
                });
            },
        }

        /** title **/
        $.ui.title = {
            setTitle: function(text) {
                $.callNativeHandler('ui.title.setTitle', {
                    title: text
                });
            },

            /**
             * 设置可点击 title
             * @param options {object}
             *        options 参数描述
             *        @tag      {int}     tag 每个页面唯一不可重复，用于点击回调响应  必填
             *        @text     {string}  标题文本，不传时，默认为当前网页 document.title
             *        @icon     {string}  标题 icon，不传时不展示, 传递可为本地预留图片或网络图片。
             *                            本地预留图片名称 "ic_title_arrow_down",
             *        @iconPos  {int}     0 icon 在标题右侧，1则在左侧。可选，不传时默认为0.
             *        @callback {function}  点击title时执行的函数，可选。
             */
            setClickableTitle: function(options) {
                var _options = {
                    tag: 1,
                    text: '',
                    icon: '',
                    iconPos: 0,
                    callback: null,
                    onSuccess: function() {},
                    onFail: function() {}
                };
                options = _extend(_options, options || {});
                var callbackId;
                if (options.callback && typeof options.callback == 'function') {
                    callbackId = options.tag;
                    messageHandlers[callbackId] = options.callback;
                }
                $.callNativeHandler('ui.title.setClickableTitle', options, function(res){
                    if (res.errno == 0) {
                        options.onSuccess();
                    }
                    else {
                        var errObj = {};
                        errObj.errno = res.errno;
                        errObj.errmsg = res.errmsg;
                        options.onFail(errObj);
                    }
                });
            },

            /**
             *  设置分段式 title
             *
             *  @param options {object}
             *      @tag       {int}             tag 每个页面唯一不可重复，用于点击回调响应  必填
             *      @segments  {array<string>}   分段按钮文本，eg. ["机票订单", "酒店订单"]
             *      @selected  {int}             初始化选中的 index, 从0开始
             *      @tintColor {string}          控件颜色，HEX值 eg. '#505050', 为空时，客户端根据自身样式展示默认颜色
             *      @callback  {function}        点击按钮回调，使用 res.segmentIndex 读取点击的按钮
             *      @onSuccess {function}        设置分段式标题成功回调，可选
             */
            setSegmentTitle: function(options) {
                var _options = {
                    tag: 1,
                    tintColor: '',
                    segments: [],
                    selected: 0,
                    callback: function() {},
                    onSuccess: function() {},
                    onFail: function() {}
                };
                options = _extend(_options, options || {});
                var callbackId;
                if (options.callback && typeof options.callback == 'function') {
                    callbackId = options.tag;
                    messageHandlers[callbackId] = options.callback;
                }
                $.callNativeHandler('ui.title.setSegmentTitle', options, function(res){
                    if (res.errno == 0) {
                        options.onSuccess();
                    }
                    else {
                        var errObj = {};
                        errObj.errno = res.errno;
                        errObj.errmsg = res.errmsg;
                        options.onFail(errObj);
                    }
                });
            },

            /**
             * 添加title右侧功能按钮
             * @param options {object}
             *        options参数描述:
             *        @tag       {int}      tag 每个页面唯一不可重复，用于点击回调响应  必填，填写重复 tag，本地不会添加新的按钮
             *        @text      {string}   icon文本       可选，不传时只展示icon按钮
             *        @icon      {string}   icon图片名称    可选，不传时只展示文本按钮
             *        @keepOrg   {int}      保留原右上角浏览器更多按钮的数量，保留时新增的按钮放到原按钮左侧，不传时默认为 -1
             *                              -1, 保留所有右侧按钮
             *                              0， 替换右侧所有按钮
             *                              1， 最多保留右侧1个按钮 ...
             *        @callback  {function} 回调函数，点击按钮执行的函数 可选
             */
            addActionButton: function(options) {
                var _options = {
                    icon: '',
                    text: '',
                    keepOrg: 0,
                    callback: function() {},
                    onSuccess: function() {},
                    onFail: function() {}
                };
                options = _extend(_options, options || {});

                var callbackId;
                if (options.callback && typeof options.callback == 'function') {
                    callbackId = options.tag;
                    messageHandlers[callbackId] = options.callback;
                }
                $.callNativeHandler('ui.title.addActionButton', options, function(res){
                    if (res.errno == 0) {
                        options.onSuccess();
                    }
                    else {
                        var errObj = {};
                        errObj.errno = res.errno;
                        errObj.errmsg = res.errmsg;
                        options.onFail(errObj);
                    }
                });
            },

            removeBtnAll: function(options) {
                var _options = {
                    onSuccess: function() {},
                    onFail: function() {}
                }
                options = _extend(_options, options || {});
                $.callNativeHandler('ui.title.removeBtnAll', options, function(res){
                    if (res.errno == 0) {
                        options.onSuccess();
                    }
                    else {
                        var errObj = {};
                        errObj.errno = res.errno;
                        errObj.errmsg = res.errmsg;
                        options.onFail(errObj);
                    }
                });
            },

            removeBtnByTag: function(options) {
                var _options = {
                    tag: 0,
                    onSuccess: function() {},
                    onFail: function() {}
                }
                options = _extend(_options, options || {});
                $.callNativeHandler('ui.title.removeBtnByTag', options, function(res){
                    if (res.errno == 0) {
                        options.onSuccess();
                    }
                    else {
                        var errObj = {};
                        errObj.errno = res.errno;
                        errObj.errmsg = res.errmsg;
                        options.onFail(errObj);
                    }
                });
            },
        };
    })(XXJS);

    /* 监听页面DOM+设备环境等信息是否Ready */
    function _isReady() {
        if (!_isDeviceReady || !_isAccountReady || !_isLocationReady || !_isEnvReady) {
            setTimeout(_isReady, 20);
        } else {
            XXJS._isAllReady = true;

            var readyEvent = document.createEvent('Events');
            readyEvent.initEvent('XXJSReady');
            document.dispatchEvent(readyEvent);
        }
    }
    _isReady();

    /**
     * 头部按钮(标题、右侧按钮)的响应事件
     */
    function _registerTitleBtnTappedHandler() {
        XXJS.registerJSHandler('title.btnTapped', function(data, responseCallback) {
            try {
                var callbackId = data['callbackId'];
                var callback = messageHandlers[callbackId];
                if (!callback) {
                    return
                };
                callback(data);
                responseCallback();
            } catch (e) {
                alert(e);
            }
        });
    }
    _registerTitleBtnTappedHandler();

    /**
     * 当页面加载完毕后执行，使用方法：
     * XXJSReady  = function(){
     *
     * };
     */
    /*  var ready = function() {
            if (typeof window.xxjsReady === 'function' && XXJS._isDeviceReady && XXJS._isAccountReady && XXJS._isLocationReady && XXJS._isEnvReady) {
                window.bnjsReady();
            } else {
                setTimeout(ready, 20);
            }
        }
        ready();
     */
})(window);
